
;;  Implements an interface to external applications
;;  This package provides two top-level functions to call external
;;  applications from Emacs.
;;  In both cases, a user function is called when new output is available
;;  (the way the output is passed to the function depends on whether this
;;  is a synchronous or asynchronous process).
;;  In both cases, the buffer user for the output of the process can be
;;  automatically deleted.


(defvar glide-external-asynchronous-func nil
  "Function to be called every time a process as output some new string.
This buffer is always buffer local.")
(make-variable-buffer-local 'glide-external-asynchronous-func)

(defvar glide-external-asynchronous-do-output nil
  "Set to true if the process output should be saved in a buffer. This
variable is always buffer local.")
(make-variable-buffer-local 'glide-external-asynchronous-do-output)

;;------------------------------
;; glide-external-synchronous --
;;------------------------------

(defun glide-external-synchronous (cmd-name param-list func
					    &optional keep-buffer)
  "Starts a synchronous process, and waits for its output before
continuing. Returns the exit status of the external application.
CMD-NAME is the name of the external program.
PARAM-LIST is the list of all the parameters to pass to the application.
The first parameter does not have to be the name of the external
application.
FUNC (if not nil) is called when the process has finish running. The current
buffer is then the output buffer, and the point is located at the top of
that buffer. It has no argument.
If KEEP-BUFFER is non-nil, the output of the process is not kept when the
process has finished running."

  (let* ((buffer (generate-new-buffer " *glide-external*"))
	 exit-status)

    ;;  Start the external process, and wait for its completion
    (set 'exit-status
	 (eval (append
		(list 'call-process
		      cmd-name  ;; Command name
		      nil       ;; No input from external file
		      buffer    ;; Output inserted in buffer
		      nil)      ;; Don't redisplay buffer when inserting output
		param-list)))

    ;;  If the process was started correctly and there is a function to call.
    (if (and exit-status func)
	(save-excursion
	  (set-buffer buffer)
	  (widen)
	  (goto-line (point-min))
	  (funcall func)))
    (unless keep-buffer
      (kill-buffer buffer))
    exit-status))

;;-------------------------------
;; glide-external-asynchronous --
;;-------------------------------

(defun glide-external-asynchronous (cmd-name param-list func do-output
					     &optional keep-buffer)
  "Starts an asynchronous process.
This function returns the name of the external process created.
The text is output in an emacs buffer only if DO-OUTPUT is not nil. You can
get the name of the buffer with a call to `process-buffer'.
FUNC is called each time one or more new lines are added to the buffer.
The only argument to FUNC is the string that was just output by the process.
CMD-NAME is the name of the external program.
PARAM-LIST is the list of all the parameters to pass to the application. The
first parameter does not have to be the name of the external application.
If KEEP-BUFFER is non-nil, the output of the process is not kept when the
process has finished running."

  (let* ((buffer (generate-new-buffer " *glide-external*"))
	 (process-connection-type nil) ;; use a pipe for the subprocess
	 proc)

    (save-excursion
      (set-buffer buffer)

      ;;  This buffer can never be deleted
      (make-local-variable 'kill-buffer-query-functions)
      (set 'kill-buffer-query-functions
	   (append '((lambda() nil)) kill-buffer-query-functions))
      
      (setq glide-external-asynchronous-func       func
	    glide-external-asynchronous-do-output  do-output))
    
    ;;  Start the external process, and wait for its completion
    (set 'proc (eval (append
		      (list 'start-process
			    cmd-name  ;; process name
			    buffer    ;; Output inserted in buffer
			    cmd-name) ;; Command name
		      param-list)))
    (set-process-filter   proc 'glide-external-process-filter)

    (unless keep-buffer
      (set-process-sentinel proc 'glide-external-process-sentinel))
    proc))

;;-----------------------------------------------------
;; glide-external-process-filter  (private function) --
;;-----------------------------------------------------

(defun glide-external-process-filter (proc string)
  "Process filter used for glide-external."

  ;;  If the buffer is still alive
  (let ((buffer (process-buffer proc))
	do-output
	func)
    
    (if (buffer-name buffer)
	(progn
	  ;; so as to preserve the point in at the end of this function
	  (with-current-buffer buffer
	    
	    (setq do-output glide-external-asynchronous-do-output
		  func      glide-external-asynchronous-func)
	  
	    ;;  If required, save the output
	    (if do-output
		(progn
		  (save-excursion
		    ;; Insert the text, advancing the process marker.
		    (goto-char (process-mark proc))
		    (insert string)
		    (set-marker (process-mark proc) (point)))
		  
		  ;;  Move the point at the end of the process's output
		  (goto-char (process-mark proc)))))
	  
	  ;;  If required, call the user's function
	  (if func
	      (funcall func string))
	  ))))

;;------------------------------------------------------
;; glide-external-process-sentinel (private function) --
;;------------------------------------------------------

(defun glide-external-process-sentinel (proc msg)
  "Process sentinel used for glide-external.
It kills the process buffer when the process is exited."

  (let ((status (process-status proc)))
    (if (memq status '(signal stop exit closed))
	(kill-buffer (process-buffer proc)))))


(provide 'glide-external)